@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: gtkvideo.h,v 1.19 2005/10/14 18:04:22 dsalt Exp $
+ * $Id: gtkvideo.h,v 1.23 2006/04/04 21:58:54 dsalt Exp $
  *
  * a gtk xine video widget
  */
@@ -82,12 +82,16 @@ enum {
 					  gint height);
   void       gtk_video_rescale           (GtkVideo *gtv, double scale);
   xine_video_port_t *gtk_video_get_port  (GtkVideo *gtv);
+  void gtk_video_reshow (GtkVideo *);
   gboolean gtk_video_is_fullscreen (GtkVideo *gtv) ;
   void gtk_video_set_fullscreen (GtkVideo *gtv, gboolean fullscreen);
     /* note: up to the caller to make sticky etc.; use window-state-event */
   void gtk_video_make_transient_for_fullscreen (GtkVideo *gtv, GtkWindow *);
+  const GtkRequisition *gtk_video_get_fullscreen_geometry (GtkVideo *gtv);
   gboolean gtk_video_get_auto_resize (GtkVideo *gtv) ;
   void gtk_video_set_auto_resize (GtkVideo *gtv, gboolean resize);
+  void gtk_video_deny_shrink (GtkVideo *gtv);
+  gboolean gtk_video_allow_shrink (GtkVideo *gtv);
 
   gboolean gtk_video_select_vis (GtkVideo *, const char *id,
 				 xine_audio_port_t **);
