@@ -13,10 +13,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: mediamarks.c,v 1.63 2005/11/10 18:55:29 dsalt Exp $
+ * $Id: mediamarks.c,v 1.69 2006/04/08 21:34:50 dsalt Exp $
  *
  * mediamarks - load
  *            - save
@@ -43,6 +43,7 @@
 #include "play_item.h"
 
 static GtkTreeStore *mm_store;
+static GtkTreeModel *mm_model;
 static GtkTreeSelection *sel;
 static GtkWidget    *manage_dlg, *cat_dlg, *cat_entry, *mm_tree;
 static GtkTreeIter   cat_iter;
@@ -52,8 +53,31 @@ static GtkUIManager *mm_ui;
 static GtkActionGroup *media_group = NULL;
 static guint	       media_merge;
 
+#define COLUMN_TITLE		0
+#define COLUMN_MRL		1
+#define COLUMN_PLAY_ITEM	2
+
 static void mm_add_at_selected (play_item_t *play_item);
 
+static inline const char *peek_title (GtkTreeIter *iter)
+{
+  static GValue v = {};
+  if (G_IS_VALUE (&v))
+    g_value_unset (&v);
+  gtk_tree_model_get_value (mm_model, iter, COLUMN_TITLE, &v);
+  return g_value_peek_pointer (&v);
+}
+
+static inline play_item_t *peek_play_item (GtkTreeIter *iter)
+{
+  GValue v = {};
+  gtk_tree_model_get_value (mm_model, iter,
+			    COLUMN_PLAY_ITEM, &v);
+  void *p = g_value_peek_pointer (&v);
+  g_value_unset (&v);
+  return p;
+}
+
 static void menu_cb (GtkAction* action, gpointer data)
 {
   if (data)
@@ -64,24 +88,14 @@ static int gen_actions_sub (GtkUIManager *ui, GtkActionGroup *actions,
 			    GtkTreeIter *iter, int count)
 {
   do {
-    GValue v = {0};
-    char  *id;
-
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 0, &v);
-    id = g_value_peek_pointer (&v);
-
+    const char *id = peek_title (iter);
     if (id) {
       char *name = g_strdup_printf ("_mediamark_%d", count++);
       GtkAction *action = gtk_action_new (name, id, NULL, NULL);
-      play_item_t *item;
 
       gtk_action_group_add_action (actions, action);
       free (name);
-
-      g_value_unset (&v);
-      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 2, &v);
-      item = g_value_peek_pointer (&v);
-      
+      play_item_t *item = peek_play_item (iter);
       if (item)
 	g_signal_connect (G_OBJECT(action), "activate",
 			  G_CALLBACK(menu_cb), (gpointer) item);
@@ -93,8 +107,6 @@ static int gen_actions_sub (GtkUIManager *ui, GtkActionGroup *actions,
 	  count = gen_actions_sub (ui, actions, &children, count);
       }
     }
-
-    g_value_unset (&v);
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
 
   return count;
@@ -114,20 +126,9 @@ static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
 		      guint merge, int count)
 {
   do {
-    GValue v = {0};
-    char  *id;
-
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 0, &v);
-    id = g_value_peek_pointer (&v);
-
-    if (id) {
+    if (peek_title (iter)) {
       char *name = g_strdup_printf ("_mediamark_%d", count++);
-      play_item_t *item;
-
-      g_value_unset (&v);
-      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), iter, 2, &v);
-      item = g_value_peek_pointer (&v);
-      
+      play_item_t *item = peek_play_item (iter);
       if (item)
 	gtk_ui_manager_add_ui (ui, merge, path, name, name,
 			       GTK_UI_MANAGER_MENUITEM, FALSE); 
@@ -145,8 +146,6 @@ static int gen_items (GtkUIManager *ui, const char *path, GtkTreeIter *iter,
 	}
       }
     }
-
-    g_value_unset (&v);
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
 
   return count;
@@ -166,9 +165,9 @@ static void update_menu (void)
 
   /* add current items */
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter))
+  if (gtk_tree_model_get_iter_first (mm_model, &iter))
   {
-    int i;
+    unsigned int i;
     const char *tree;
     media_merge = gtk_ui_manager_new_merge_id (ui);
     media_group = gen_actions (ui, &iter);
@@ -176,7 +175,7 @@ static void update_menu (void)
     for (i = 0; (tree = menu_get_tree_name (i)) != NULL; ++i)
     {
       char *key = g_strdup_printf ("/ui/%s/MediaMenu/MediaItems", tree);
-      gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter);
+      gtk_tree_model_get_iter_first (mm_model, &iter);
       gen_items (ui, key, &iter, media_merge, 0);
       free (key);
     }
@@ -199,7 +198,7 @@ static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
       id = xml_parser_get_property (node, "name");
 
       gtk_tree_store_append (mm_store, &iter, parent);
-      gtk_tree_store_set (mm_store, &iter, 0, id, -1);
+      gtk_tree_store_set (mm_store, &iter, COLUMN_TITLE, id, -1);
 
       get_items (node->child, depth+1, &iter);
 
@@ -208,11 +207,14 @@ static void get_items (xml_node_t *node, int depth, GtkTreeIter *parent) {
       GtkTreeIter  iter;
 
       gtk_tree_store_append (mm_store, &iter, parent);
-      gtk_tree_store_set (mm_store, &iter, 0, play_item->title, 1, 
-			  play_item->mrl, 2, play_item, -1);
+      gtk_tree_store_set (mm_store, &iter,
+			  COLUMN_TITLE, play_item->title,
+			  COLUMN_MRL, play_item->mrl,
+			  COLUMN_PLAY_ITEM, play_item,
+			  -1);
 
     } else {
-      fprintf (stderr, _("mediamarks: error: unknown node type %s\n"), node->name);
+      g_printerr (_("mediamarks: error: unknown node type %s\n"), node->name);
     }
 }
 
@@ -222,7 +224,7 @@ static void load_new_file (xml_node_t *node) {
 
     get_items (node->child, 0, NULL);
   } else {
-    fprintf (stderr, _("mediamarks: error: root node must be GXINEMM\n"));
+    g_printerr (_("mediamarks: error: root node must be GXINEMM\n"));
   }
 }
 
@@ -253,7 +255,7 @@ static void load_old_file (char *fname) {
   GtkTreeIter  parent;
   GtkTreeIter  iter;
 
-  gtk_tree_model_get_iter_first ( GTK_TREE_MODEL (mm_store), &parent);
+  gtk_tree_model_get_iter_first (mm_model, &parent);
 
   f = fopen (fname, "r");
   if (f)
@@ -269,7 +271,7 @@ static void load_old_file (char *fname) {
       if (!strncmp (id, "<sub>", 5))
       {
 	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
-	gtk_tree_store_set (mm_store, &iter, 0, &id[5], -1);
+	gtk_tree_store_set (mm_store, &iter, COLUMN_TITLE, &id[5], -1);
 
 	parent = iter;
 	depth++;
@@ -277,11 +279,11 @@ static void load_old_file (char *fname) {
       else if (!strncmp (id, "</sub>", 6))
       {
 	/* sub-tree ends here */
-	gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store), 
+	gtk_tree_model_iter_parent (mm_model, 
 				    &iter, &parent);
 	if (--depth < 0)
 	{
-	  fprintf (stderr, _("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
+	  g_printerr (_("mediamarks: media marks file %s corrupted - ignoring rest of file\n"),
 		   fname);
 	  free (id);
 	  return;
@@ -312,7 +314,11 @@ static void load_old_file (char *fname) {
 	play_item = play_item_new (id, mrl, time);
 
 	gtk_tree_store_append (mm_store, &iter, depth ? &parent : NULL);
-	gtk_tree_store_set (mm_store, &iter, 0, id, 1, mrl, 2, play_item, -1);
+	gtk_tree_store_set (mm_store, &iter,
+			    COLUMN_TITLE, id,
+			    COLUMN_MRL, mrl,
+			    COLUMN_PLAY_ITEM, play_item,
+			    -1);
 
 	free (str);
 	free (mrl);
@@ -328,7 +334,7 @@ static void load_old_file (char *fname) {
     fclose (f);
   }
   else
-    fprintf (stderr, _("mediamarks: load failed!\n"));
+    g_printerr (_("mediamarks: load failed!\n"));
 }
 
 static void cat_response_cb (GtkDialog* widget, int response, gpointer data)
@@ -361,12 +367,7 @@ static void edit_cb (GtkWidget* widget, gpointer data)
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
   {
-    GValue       v = {0};
-    play_item_t *play_item;
-
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      &iter, 2, &v);
-    play_item = g_value_peek_pointer (&v);
+    play_item_t *play_item = peek_play_item (&iter);
     if (play_item) {
       logprintf ("mediamarks: got a play item\n");
       if (play_item_edit (play_item, PLAY_ITEM_MEDIAMARK, NULL, manage_dlg))
@@ -374,26 +375,18 @@ static void edit_cb (GtkWidget* widget, gpointer data)
 	logprintf ("mediamarks: play item title after edit : %s\n",
 		   play_item->title);
 	gtk_tree_store_set (mm_store, &iter,
-			    0, play_item->title,
-			    1, play_item->mrl, 2, play_item, -1);
+			    COLUMN_TITLE, play_item->title,
+			    COLUMN_MRL, play_item->mrl,
+			    COLUMN_PLAY_ITEM, play_item,
+			    -1);
       }
     } else {
-      GValue       v = {0};
-      char        *name;
-
       memcpy (&cat_iter, &iter, sizeof (cat_iter));
-
-      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-				&iter, 0, &v);
-      name = (char *) g_value_peek_pointer (&v);
-      gtk_entry_set_text (GTK_ENTRY (cat_entry), name);
-      g_value_unset (&v);
-
+      gtk_entry_set_text (GTK_ENTRY (cat_entry), peek_title (&iter));
       if (do_category_edit ())
-	gtk_tree_store_set (mm_store, &cat_iter, 0,
+	gtk_tree_store_set (mm_store, &cat_iter, COLUMN_TITLE,
 			    gtk_entry_get_text (GTK_ENTRY(cat_entry)), -1);
     }
-    g_value_unset (&v);
   }
 }
 
@@ -431,35 +424,30 @@ static void new_category_cb (GtkWidget* widget, gpointer data)
    * find out where to create the new section
    */
 
-  if (gtk_tree_selection_get_selected (sel, NULL, &iter)) {
-    GValue       v = {0};
-    play_item_t *play_item;
-
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
     /*
      * is this a section or a play item ?
      */
-
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      &iter, 2, &v);
-    play_item = g_value_peek_pointer (&v);
-    if (play_item) {
-      in_root = !gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store),
+    if (peek_play_item (&iter))
+      in_root = !gtk_tree_model_iter_parent (mm_model,
 					     &parent, &iter);
-    } else {
+    else
+    {
       GtkTreePath *path;
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (mm_store), &iter);
+      path = gtk_tree_model_get_path (mm_model, &iter);
 
-      if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (mm_tree), path)) {
+      if (gtk_tree_view_row_expanded (GTK_TREE_VIEW (mm_tree), path))
+      {
 	in_root = 0;
 	memcpy (&parent, &iter, sizeof (iter));
-      } else {
-	in_root = !gtk_tree_model_iter_parent (GTK_TREE_MODEL (mm_store),
-					       &parent, &iter);
       }
+      else
+	in_root = !gtk_tree_model_iter_parent (mm_model,
+					       &parent, &iter);
 
       gtk_tree_path_free (path);
     }
-    g_value_unset (&v);
   }
 
   /*
@@ -472,7 +460,7 @@ static void new_category_cb (GtkWidget* widget, gpointer data)
   {
     /* create new category */
     gtk_tree_store_append (mm_store, &cat_iter, in_root ? NULL : &parent);
-    gtk_tree_store_set (mm_store, &cat_iter, 0,
+    gtk_tree_store_set (mm_store, &cat_iter, COLUMN_TITLE,
 			gtk_entry_get_text (GTK_ENTRY(cat_entry)), -1);
   }
 }
@@ -493,10 +481,13 @@ static int load_mediamarks (gchar *fname)
     xml_parser_init (mmfile, strlen (mmfile), XML_PARSER_CASE_INSENSITIVE);
     g_signal_handlers_block_by_func (G_OBJECT(mm_store), tree_changed_cb, NULL);
 
-    if (xml_parser_build_tree (&node)<0) {
+    if (xml_parser_build_tree (&node) < 0)
       load_old_file (fname);
-    } else 
+    else
+    {
       load_new_file (node);
+      xml_parser_free_tree (node);
+    }
 
     update_menu ();
     g_signal_handlers_unblock_by_func (G_OBJECT(mm_store), tree_changed_cb, NULL);
@@ -557,8 +548,11 @@ void mm_add (play_item_t *play_item)
 {
   GtkTreeIter iter;
   gtk_tree_store_append (mm_store, &iter, NULL);
-  gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
-		      1, play_item->mrl, 2, play_item, -1);
+  gtk_tree_store_set (mm_store, &iter,
+		      COLUMN_TITLE, play_item->title,
+		      COLUMN_MRL, play_item->mrl,
+		      COLUMN_PLAY_ITEM, play_item,
+		      -1);
 }
 
 static void mm_add_at_selected (play_item_t *play_item)
@@ -566,16 +560,16 @@ static void mm_add_at_selected (play_item_t *play_item)
   GtkTreeIter iter, selected;
   if (gtk_tree_selection_get_selected (sel, NULL, &selected))
   {
-    GValue v = {0};
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), &selected, 2, &v);
-    if (g_value_peek_pointer (&v) == NULL)
+    if (!peek_play_item (&selected))
       gtk_tree_store_insert_after (mm_store, &iter, &selected, NULL);
     else
       gtk_tree_store_insert_after (mm_store, &iter, NULL, &selected);
 
-    gtk_tree_store_set (mm_store, &iter, 0, play_item->title,
-		        1, play_item->mrl, 2, play_item, -1);
-    g_value_unset (&v);
+    gtk_tree_store_set (mm_store, &iter,
+			COLUMN_TITLE, play_item->title,
+			COLUMN_MRL, play_item->mrl,
+			COLUMN_PLAY_ITEM, play_item,
+			-1);
   }
   else
     mm_add (play_item);
@@ -593,35 +587,19 @@ void mm_manage_show (void)
   }
 }
 
-static void print_depth (FILE *f, int depth) {
-  depth += 2;
-
-  while (depth) {
-    fprintf (f, " ");
-    depth --;
-  }
+static inline void print_depth (FILE *f, int depth)
+{
+  fprintf (f, "%*s  ", depth, "");
 }
 
 static void save_items (FILE *f, int depth, GtkTreeIter *iter)
 {
   do
   {
-    GValue        v = {0};
-    play_item_t  *play_item;
-    char         *id;
-      
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-			      iter, 0, &v);
-    id = g_value_peek_pointer (&v);
-
+    const char *id = peek_title (iter);
     if (id)
     {
-      GValue vi = {0};
-
-      gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store),
-				iter, 2, &vi);
-      play_item = g_value_peek_pointer (&vi);
-      
+      play_item_t *play_item = peek_play_item (iter);
       if (play_item)
       {
         char *xml = play_item_xml (play_item, depth);
@@ -641,47 +619,29 @@ static void save_items (FILE *f, int depth, GtkTreeIter *iter)
 	print_depth (f, depth);
 	fprintf (f, "</SUB>\n");
       }
-      
-      g_value_unset (&vi);
     }
-
-    g_value_unset (&v);
       
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(mm_store), iter));
 }
 
-void mm_save (void) {
-
-  FILE    *f;
-  gchar   *fname;
-
-  fname = g_strconcat(g_get_home_dir(), "/.gxine/mediamarks", NULL);
-
-  f = fopen (fname, "w") ;
-
-  if (f) {
+void mm_save (void)
+{
+  gchar *fname = g_strconcat(g_get_home_dir(), "/.gxine/mediamarks", NULL);
+  FILE *f = open_write (fname, _("Failed to save media marks"));
 
+  if (f)
+  {
     GtkTreeIter iter;
 
     fprintf (f, "<GXINEMM VERSION=\"1\">\n");
 
-    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (mm_store), &iter))
+    if (gtk_tree_model_get_iter_first (mm_model, &iter))
       save_items (f, 0, &iter);
 
     fprintf (f, "</GXINEMM>\n");
 
-    if (ferror (f))
-      display_error_modal (FROM_GXINE, _("Failed to save media marks"),
-			   _("Error while writing to '%s': %s"),
-			   fname, strerror (errno));
-    if (fclose (f))
-      display_error_modal (FROM_GXINE, _("Failed to save media marks"),
-			   _("Error when closing '%s': %s"),
-			   fname, strerror (errno));
-  } else 
-    display_error_modal (FROM_GXINE, _("Failed to save media marks"),
-                         _("Can't open file '%s': %s"),
-			 fname, strerror (errno));
+    close_write (fname, f, _("Failed to save media marks"));
+  }
 
   g_free(fname);
 }
@@ -758,12 +718,7 @@ static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
 
   ui_mark_active (mm_ui, items, have_selected);
   if (have_selected)
-  {
-    GValue v = {0};
-    gtk_tree_model_get_value (GTK_TREE_MODEL (mm_store), &iter, 2, &v);
-    ui_mark_active (mm_ui, cb_items, g_value_peek_pointer (&v) != NULL);
-    g_value_unset (&v);
-  }
+    ui_mark_active (mm_ui, cb_items, peek_play_item (&iter) != NULL);
   else
     ui_mark_active (mm_ui, cb_items, FALSE);
 }
@@ -782,6 +737,7 @@ void mediamarks_init (void)
    */
  
   mm_store = gtk_tree_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+  mm_model = GTK_TREE_MODEL (mm_store);
   g_object_connect (G_OBJECT (mm_store),
 	"signal::row-changed", G_CALLBACK (tree_changed_cb), NULL,
 	"signal::row-deleted", G_CALLBACK (tree_changed_cb), NULL,
@@ -832,7 +788,7 @@ void mediamarks_init (void)
   g_object_set (G_OBJECT(cell),
 		"ellipsize", 2, "width_chars", 16, NULL); /* foo...bar */
   column = gtk_tree_view_column_new_with_attributes (_("Media mark"), cell,
-						     "text", 0, NULL);
+						     "text", COLUMN_TITLE, NULL);
   gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
 			       GTK_TREE_VIEW_COLUMN (column));
@@ -841,7 +797,7 @@ void mediamarks_init (void)
   g_object_set (G_OBJECT(cell),
 		"ellipsize", 1, "width_chars", 16, NULL); /* ...bar */
   column = gtk_tree_view_column_new_with_attributes (_("MRL"), cell,
-						     "text", 1, NULL);
+						     "text", COLUMN_MRL, NULL);
   gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
 				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (tree),
