@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: menu.h,v 1.7 2005/07/13 22:55:59 dsalt Exp $
+ * $Id: menu.h,v 1.8 2006-02-21 19:06:12 dsalt Exp $
  *
  * main/context menu creation / handling
  */
@@ -28,7 +28,7 @@
 void create_menus (GtkWidget *window);
 void menu_add_items (GtkUIManager *, const char *const prefix,
 		     const char *const label, guint merge, size_t count);
-const char *menu_get_tree_name (int);
+const char *menu_get_tree_name (unsigned int);
 
 /* signal callback & support for dynamic scale changes */
 void scale_changed_cb (GtkWidget *, gdouble, gpointer);
