@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: noskin_window.c,v 1.90 2005/11/02 23:52:10 dsalt Exp $
+ * $Id: noskin_window.c,v 1.108 2006/04/08 21:34:50 dsalt Exp $
  *
  * standard, non-skinned main window
  */
@@ -29,6 +29,10 @@
 #include <gdk/gdkx.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
+#include <X11/Xatom.h>
 
 #include "ui.h"
 #include "utils.h"
@@ -40,8 +44,6 @@
 #include "menu.h"
 #include "drag_drop.h"
 #include "gtkvideo.h"
-#include "gtkflipbutton.h"
-#include "info_widgets.h"
 #include "vis.h"
 #include "xml_widgets.h"
 
@@ -56,8 +58,12 @@ GtkWidget *gtv;
 /* private/local */
 static GtkTooltips    *tips;
 static int             have_video = -1, have_audio = -1;
+static struct app_prop_s {
+  gboolean	       sticky, fs, real_sticky;
+} app_prop = { FALSE, FALSE, FALSE };
 
 static GtkWidget *fs_toolbar, *wm_toolbar = NULL;
+static Atom opacity_lock;
 
 static void gtk_window_set_shade (GtkWindow *w, gboolean shade)
 {
@@ -87,9 +93,14 @@ static void gtk_window_set_shade (GtkWindow *w, gboolean shade)
 
 void window_check_vis (gboolean force)
 {
+  xine_cfg_entry_t entry;
   int hv_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_VIDEO);
+  if (!have_video && playlist_showing_logo ()
+      && xine_config_lookup_entry (xine, "gui.post_plugins.always_show_logo", &entry)
+      && !entry.num_value)
+    hv_new = 0;
   int ha_new = xine_get_stream_info (stream, XINE_STREAM_INFO_HAS_AUDIO);
-  gboolean fs = gtk_video_is_fullscreen (GTK_VIDEO(gtv));
+  gboolean fs = gtk_video_is_fullscreen ((GtkVideo *)gtv);
   gboolean was_shown = GTK_WIDGET_DRAWABLE (gtv);
   gboolean is_shown = was_shown;
 
@@ -99,13 +110,13 @@ void window_check_vis (gboolean force)
     have_audio = ha_new;
     if (hv_new) {
       logprintf ("gxine: stream has video\n");
-      vis_hide (GTK_VIDEO(gtv), &audio_port);
+      vis_hide ((GtkVideo *)gtv, &audio_port);
 #ifdef USE_NET_WM_STATE_SHADED
       if (wm_toolbar)
 	gtk_window_set_shade (GTK_WINDOW (app), FALSE);
       else
 #endif
-	gtk_widget_show (gtv);
+	gtk_video_reshow ((GtkVideo *)gtv);
       is_shown = TRUE;
     }
     else if (!ha_new)
@@ -113,7 +124,7 @@ void window_check_vis (gboolean force)
       logprintf ("gxine: stream has no audio\n");
       /* stream probably not initialised yet - don't show or hide */
     }
-    else if (vis_show (GTK_VIDEO(gtv), &audio_port))
+    else if (vis_show ((GtkVideo *)gtv, &audio_port))
     {
       logprintf ("gxine: stream has audio (visualisation active)\n");
 #ifdef USE_NET_WM_STATE_SHADED
@@ -121,7 +132,7 @@ void window_check_vis (gboolean force)
 	gtk_window_set_shade (GTK_WINDOW (app), FALSE);
       else
 #endif
-	gtk_widget_show (gtv);
+	gtk_video_reshow ((GtkVideo *)gtv);
       is_shown = TRUE;
     }
     else
@@ -142,15 +153,15 @@ void window_check_vis (gboolean force)
     }
     if (action_items.fullscreen)
       gtk_action_set_sensitive ((GtkAction *)action_items.fullscreen, is_shown);
-    /* Kludge: resize if the video widget visibility has been changed.
-     * If you have a better way of doing this, submit a patch...
+    /* Resize if the video widget visibility has been changed.
+     * Note that the GtkVideo widget imposes a geometry constraint...
      */
 #ifdef USE_NET_WM_STATE_SHADED
     if (!fs && !wm_toolbar && is_shown != was_shown)
-      gtk_window_resize (GTK_WINDOW(app), 10, 10);
+      gtk_window_resize (GTK_WINDOW(app), GTK_VIDEO_MIN_WIDTH, 1);
 #else
     if (!fs && is_shown != was_shown)
-      gtk_window_resize (GTK_WINDOW(app), 10, 10);
+      gtk_window_resize (GTK_WINDOW(app), GTK_VIDEO_MIN_WIDTH, 1);
 #endif
   }
 }
@@ -175,7 +186,8 @@ static gboolean update_slider_cb (gpointer data)
   return TRUE;
 }
 
-gint close_application (GtkWidget *widget, GdkEvent *event, gpointer data)
+static gint
+close_application (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
   engine_exec ("exit();", NULL, NULL, NULL);
   return FALSE;
@@ -192,9 +204,10 @@ static gboolean window_fs_toolbar_move (GdkEventConfigure *event)
   int h = fs_toolbar_at_top
 	  ? 1
 	  : (event ? event->height : app->allocation.height) - cw_height - 1;
+  int x, y;
+  gtk_window_get_position (GTK_WINDOW(app), &x, &y);
   gtk_window_move (GTK_WINDOW(fs_toolbar),
-		   event ? event->x : app->allocation.x,
-		   (event ? event->y : app->allocation.y) + h);
+		   event ? event->x : x, (event ? event->y : y) + h);
   return FALSE;
 }
 
@@ -203,6 +216,10 @@ static gboolean window_fs_toolbar_show_int (void)
   window_fs_toolbar_move (NULL);
   window_show (fs_toolbar, NULL);
   gtk_window_stick (GTK_WINDOW(fs_toolbar));
+  XChangeProperty (GDK_WINDOW_XDISPLAY (fs_toolbar->window),
+		   GDK_WINDOW_XWINDOW (fs_toolbar->window), opacity_lock,
+		   XA_CARDINAL, 32, PropModeReplace,
+		   (unsigned char *)&have_video, 1);
   return FALSE;
 }
 
@@ -212,7 +229,7 @@ void window_fs_toolbar_show (gboolean show)
 
   if (!show)
     window_fs_toolbar_reset ();
-  else if (gtk_video_is_fullscreen (GTK_VIDEO(gtv)))
+  else if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
     window_fs_toolbar_show_int ();
 
   ui_set_status (UI_FS_TOOLBAR);
@@ -279,7 +296,7 @@ void window_wm_toolbar_show (gboolean show)
 
   if (!show)
     window_wm_toolbar_reset ();
-  else if (!gtk_video_is_fullscreen (GTK_VIDEO(gtv)))
+  else if (!gtk_video_is_fullscreen ((GtkVideo *)gtv))
     window_wm_toolbar_restore ();
 
   ui_set_status (UI_WM_TOOLBAR);
@@ -309,11 +326,23 @@ void window_wm_toolbar_set_snap (gboolean snap)
 
 /* callbacks */
 
+#ifdef WINDOW_STACKING_HACK
 /* whether gdk_window_lower() is used; present in case the wm toolbar window
  * should somehow gain the input focus, since calling this may move the focus
  * elsewhere
  */
 static gboolean can_lower = FALSE;
+static inline gboolean set_can_lower (gboolean v)
+{
+  return can_lower = v;
+}
+#else
+#define can_lower (FALSE)
+static inline gboolean set_can_lower (gboolean v)
+{
+  return v;
+}
+#endif /* !WINDOW_STACKING_HACK */
 
 gboolean window_wm_toolbar_snap (void)
 {
@@ -388,73 +417,151 @@ static void fs_show_hide (GtkWidget *widget, gpointer show)
     gtk_widget_hide (widget);
 }
 
-static gboolean app_state_cb (GtkWindow *window, GdkEventWindowState *event,
-			      gpointer data)
-{
-  static gboolean sticky = FALSE, lock = FALSE;
-
-  if (event->changed_mask & GDK_WINDOW_STATE_STICKY)
-  {
-    if (!lock)
-    {
-      if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
-	gtk_window_stick (window); /* prevent unstick in fullscreen (bug) */
-      else
-	sticky = !!(event->new_window_state & GDK_WINDOW_STATE_STICKY);
-    } 
-    lock = FALSE;
-  }
-
-  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
-  {
-    gboolean notfs = !(event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN);
-
-    if (notfs)
-    {
-      if (!sticky)
-	gtk_window_unstick (window); /* may have been stuck by us, not user */
-      gtk_window_set_keep_above (window, FALSE);
-      if (!wm_toolbar)
-	gtk_container_foreach (GTK_CONTAINER (window->bin.child),
-			       fs_show_hide, window);
-      else if (wm_toolbar_visible)
-	window_wm_toolbar_restore ();
-    }
-    else
-    {
-      lock = !sticky; /* we may be triggering a state change: lock it out */
-      gtk_window_set_shade (window, FALSE);
-      gtk_window_stick (window);
-      if (!wm_toolbar)
-	gtk_container_foreach (GTK_CONTAINER (window->bin.child),
-			       fs_show_hide, NULL);
-      else
-	window_wm_toolbar_reset ();
-      gtk_window_set_keep_above (window, TRUE);
-      XSetInputFocus (GDK_WINDOW_XDISPLAY (gtv->window),
-		      GDK_WINDOW_XWINDOW (gtv->window),
-		      RevertToParent, CurrentTime);
-    }
-    
-    gtk_toggle_action_set_active (action_items.fullscreen, !notfs);
-  }
-
-  return FALSE;
-}
-
 static gboolean refocus_cb (gpointer widget)
 {
-  gtk_widget_grab_focus (widget);
+  if (GTK_WIDGET_VISIBLE (widget))
+    gtk_widget_grab_focus (widget);
   return FALSE;
 }
 
 static gboolean appfocus_cb (GtkWidget *w, GdkEventFocus *e, gpointer d)
 {
-  puts ("Eek, toolbar has focus");
+  g_print (_("Eek, toolbar has focus"));
   g_idle_add (refocus_cb, app);
   return TRUE;
 }
 
+static gboolean
+app_state_cb (GtkWindow *window, GdkEventWindowState *event, gpointer data)
+{
+  if (!(event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN))
+    return FALSE;
+
+  int fs = event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN;
+
+  if (!fs)
+  {
+    gtk_video_deny_shrink ((GtkVideo *) gtv);
+
+    /* restore stickiness as was, pre-fullscreen */
+    if (app_prop.sticky)
+      gtk_window_stick (window);
+    else
+      gtk_window_unstick (window);
+
+    /* restore handling wrt window stack */
+    gtk_window_set_keep_above (window, FALSE);
+
+    /* reshow the toolbar (if needed) */
+    if (!wm_toolbar)
+    {
+      gdk_flush ();
+      gtk_container_foreach (GTK_CONTAINER (window->bin.child),
+			     fs_show_hide, window);
+      gdk_flush ();
+    }
+    else if (wm_toolbar_visible)
+      window_wm_toolbar_restore ();
+
+    gtk_toggle_action_set_active (action_items.fullscreen, fs);
+  }
+  else if (!GTK_WIDGET_VISIBLE (gtv))
+  {
+    if (playlist_showing_logo ())
+    {
+      gtk_widget_show (gtv);
+      gtk_action_set_sensitive ((GtkAction *)action_items.fullscreen, TRUE);
+    }
+    else
+      gtk_window_unfullscreen (window);
+  }
+  else
+  {
+    gtk_window_set_shade (window, FALSE);
+    gtk_window_stick (window);
+    if (!wm_toolbar)
+      gtk_container_foreach (GTK_CONTAINER (window->bin.child),
+			     fs_show_hide, NULL);
+    else
+      window_wm_toolbar_reset ();
+    gtk_window_set_keep_above (window, TRUE);
+    XSetInputFocus (GDK_WINDOW_XDISPLAY (app->window),
+		    GDK_WINDOW_XID (app->window), RevertToParent, CurrentTime);
+
+    gtk_toggle_action_set_active (action_items.fullscreen, fs);
+  }
+
+  return FALSE;
+}
+
+static void
+xevent_properties (void)
+{
+  gdk_error_trap_push ();
+
+  Display *xd = GDK_WINDOW_XDISPLAY (app->window);
+  GdkDisplay *d = gdk_x11_lookup_xdisplay (xd);
+  Window xw = GDK_WINDOW_XID (app->window);
+
+  Atom type;
+  int fmt;
+  unsigned long items, bytes;
+  unsigned char *prop;
+
+  XGetWindowProperty
+    (xd, xw, gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE"),
+     0, G_MAXLONG, False, XA_ATOM, &type, &fmt, &items, &bytes, &prop);
+
+  if (type == None)
+    goto unchanged;
+
+  struct app_prop_s new_prop = { .sticky = app_prop.sticky };
+
+  Atom sticky =
+    gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_STICKY");
+  Atom fs =
+    gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_STATE_FULLSCREEN");
+  Atom *atoms = (Atom *) prop;
+  unsigned long i;
+  for (i = 0; i < items; ++i)
+    if (atoms[i] == sticky)
+      new_prop.real_sticky = TRUE;
+    else if (atoms[i] == fs)
+      new_prop.fs = TRUE;
+
+  XFree (prop);
+
+  if (new_prop.real_sticky)
+  {
+    /* sticky only if "sticky" property set & window on all desktops */
+    XGetWindowProperty
+      (xd, xw, gdk_x11_get_xatom_by_name_for_display (d, "_NET_WM_DESKTOP"),
+       0, G_MAXLONG, False, XA_CARDINAL, &type, &fmt, &items, &bytes, &prop);
+    if (type != None)
+    {
+      new_prop.real_sticky = *(int32_t *) prop == -1;
+      XFree (prop);
+    }
+  }
+
+  if (app_prop.real_sticky != new_prop.real_sticky)
+  {
+    if (!new_prop.fs)
+      new_prop.sticky = new_prop.real_sticky;
+    else if (!new_prop.real_sticky)
+    {
+      /* prevent unstick in fullscreen */
+      new_prop.real_sticky = TRUE;
+      gtk_window_stick ((GtkWindow *)app);
+    }
+  }
+
+  app_prop = new_prop;
+
+unchanged:
+  gdk_error_trap_pop ();
+}
+
 static GdkFilterReturn
 xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
 {
@@ -474,6 +581,10 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
     if (gtk_video_is_fullscreen ((GtkVideo *) gtv))
     {
       Display *xd = GDK_WINDOW_XDISPLAY (((GtkWidget *) d)->window);
+#ifdef HAVE_XINERAMA
+      if (XineramaIsActive (xd))
+        return GDK_FILTER_REMOVE;
+#endif
       Window fw = None;
       int fr;
       XGetInputFocus (xd, &fw, &fr);
@@ -482,24 +593,16 @@ xevent_filter_cb (GdkXEvent *gx, GdkEvent *e, gpointer d)
 	XSetInputFocus (xd, x->xany.window, RevertToParent, CurrentTime);
     }
     return GDK_FILTER_REMOVE;
+
+  case PropertyNotify:
+    if (app->window)
+      xevent_properties ();
+    break;
   }
 
   return GDK_FILTER_CONTINUE;
 }
 
-static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
-			    gpointer data)
-{
-  char **js;
-  while ((js = g_async_queue_try_pop (js_queue)))
-  {
-    engine_exec (js[0], NULL, NULL, js[1]);
-    free (js[1]);
-    free (js);
-  }
-  return TRUE;
-}
-
 static gboolean noskin_main_init_cb (void)
 {
   gtk_widget_add_events (app, GDK_KEY_RELEASE_MASK);
@@ -507,20 +610,31 @@ static gboolean noskin_main_init_cb (void)
 	"signal::key-press-event", G_CALLBACK (keypress_cb), wm_toolbar,
 	"signal::popup-menu", G_CALLBACK(popup_cb), NULL,
 	NULL);
+  if (wm_toolbar)
+    g_signal_connect (G_OBJECT (wm_toolbar),
+		      "key-press-event", G_CALLBACK (keypress_cb), NULL);
+  g_signal_connect (G_OBJECT (fs_toolbar),
+		    "key-press-event", G_CALLBACK (keypress_cb), NULL);
   ui_set_status (xine_get_param (stream, XINE_PARAM_AUDIO_MUTE)
 		 ? UI_AUDIO_MUTE : UI_AUDIO_UNMUTE);
   gdk_window_set_events (app->window, GDK_ALL_EVENTS_MASK); /* seems to work */
   gdk_window_add_filter (app->window, xevent_filter_cb, app);
+  gtk_widget_grab_focus (app);
+/*
+  XSetInputFocus (GDK_WINDOW_XDISPLAY (app->window),
+		  GDK_WINDOW_XWINDOW (app->window),
+		  RevertToParent, CurrentTime);
+*/
   return FALSE;
 }
 
 /* initialisation */
 
-gboolean app_configure_cb (GtkWidget *widget, GdkEventConfigure *event,
-			   gpointer data)
+static gboolean app_configure_cb (GtkWidget *widget, GdkEventConfigure *event,
+				  gpointer data)
 {
   /* redisplay/reposition the toolbar (XRANDR support) */
-  if (gtk_video_is_fullscreen (GTK_VIDEO (gtv)))
+  if (gtk_video_is_fullscreen ((GtkVideo *)gtv))
   {
 #ifdef HAVE_XRANDR
     if (fs_toolbar_visible)
@@ -535,17 +649,42 @@ gboolean app_configure_cb (GtkWidget *widget, GdkEventConfigure *event,
     window_wm_toolbar_snap ();
     if (event->width != GTK_VIDEO_MIN_WIDTH &&
 	event->height != GTK_VIDEO_MIN_HEIGHT)
-      can_lower = TRUE; /* we've done the initial resize */
+      set_can_lower (TRUE); /* we've done the initial resize */
   }
   return FALSE;
 }
 
+static int wm_hide_count = 0;
+
+static void
+wm_show_cb (GtkWidget *w, gpointer data)
+{
+  if (wm_hide_count && !--wm_hide_count)
+    g_idle_add ((GSourceFunc) gtk_video_allow_shrink, gtv);
+}
+
+static void
+wm_hide_cb (GtkWidget *w, gpointer data)
+{
+  ++wm_hide_count;
+}
+
+static void
+wm_show_hide_setup (gpointer object)
+{
+  ++wm_hide_count;
+  g_object_connect (object,
+		    "signal::show", G_CALLBACK (wm_show_cb), NULL,
+		    "signal::hide", G_CALLBACK (wm_hide_cb), NULL,
+		    NULL);
+}
+
 static gboolean noskin_post_init (gboolean unused)
 {
   xine_cfg_entry_t entry;
   if (xine_config_lookup_entry (xine, "gui.window_size", &entry))
   {
-    static const gdouble scale[] = { 50, 100, 200 };
+    static const gdouble scale[] = { 50, 75, 100, 150, 200 };
     gtk_video_rescale ((GtkVideo *)gtv, scale[entry.num_value]);
   }
   if (xine_config_lookup_entry (xine, "gui.magnify_lowres_video", &entry))
@@ -556,12 +695,8 @@ static gboolean noskin_post_init (gboolean unused)
 
 void noskin_main_init (void)
 {
-  GtkWidget     *vbox;
+  GtkWidget     *vbox, *w;
   GtkWindow     *W;
-#ifdef VIS_WIDGET
-  GtkWidget     *vis;
-  xine_post_out_t *vis_out;
-#endif
   xine_cfg_entry_t entry;
 
   tips = gtk_tooltips_new ();
@@ -569,12 +704,18 @@ void noskin_main_init (void)
   /* create app windows */
 
   app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_widget_set_name (app, "video");
   fs_toolbar = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_widget_set_name (fs_toolbar, "fs_toolbar");
 
   if (!xine_config_lookup_entry (xine, "gui.windowedmode_separate_toolbar",
 				 &entry)
       || entry.num_value)
+  {
     wm_toolbar = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+    gtk_widget_set_name (wm_toolbar, "wm_toolbar");
+    drag_drop_setup (wm_toolbar, TRUE);
+  }
 
   /*
    * toolbar window (full-screen mode)
@@ -589,6 +730,7 @@ void noskin_main_init (void)
 
   W = GTK_WINDOW (fs_toolbar);
 
+  /* toolbar window title */
   gtk_window_set_title (W, _("gxine controls"));
   gtk_window_set_resizable (W, FALSE);
   gtk_window_set_type_hint (W, GDK_WINDOW_TYPE_HINT_TOOLBAR);
@@ -603,10 +745,13 @@ void noskin_main_init (void)
   gtk_window_set_skip_taskbar_hint (W, TRUE);
 
   window_fs_toolbar_set_geometry ();
+  drag_drop_setup (fs_toolbar, TRUE);
 
-  gtk_container_add (GTK_CONTAINER (fs_toolbar),
-		     widget_create_from_xml ("toolbar-fullscreen.xml",
-					     "toolbar_fs", NULL, FALSE));
+  w = widget_create_from_xml ("toolbar-fullscreen.xml", "toolbar_fs",
+			      NULL, FALSE, TRUE);
+  if (!w)
+    exit (2); /* no point in continuing */
+  gtk_container_add (GTK_CONTAINER (fs_toolbar), w);
 
   /*
    * video window
@@ -619,9 +764,8 @@ void noskin_main_init (void)
 	"signal::delete-event", G_CALLBACK (close_application), NULL,
 	"signal::configure-event", G_CALLBACK (app_configure_cb), NULL,
 	"signal::window-state-event", G_CALLBACK (app_state_cb), NULL,
-	"signal::client-event", G_CALLBACK (js_queue_cb), NULL,
 	NULL);
-  drag_drop_setup (app);
+  drag_drop_setup (app, TRUE);
 
   /*
    * video widget
@@ -631,7 +775,6 @@ void noskin_main_init (void)
   	               video_driver_id, 320, 240,
                        0x04 /* press: button 2 */,
                        0x08 /* release: button 3 */);
-  drag_drop_setup (gtv);
   g_object_connect (G_OBJECT (gtv),
 	/*"signal::key-press-event", G_CALLBACK (keypress_cb), wm_toolbar,*/
 	"signal::button-press-event", G_CALLBACK (buttonpress_cb), NULL,
@@ -639,7 +782,7 @@ void noskin_main_init (void)
 	"signal::scale-factor-changed", G_CALLBACK (scale_changed_cb), NULL,
 	NULL);
 
-  gtk_init_add (noskin_post_init, NULL);
+  gtk_init_add ((GSourceFunc) noskin_post_init, NULL);
 
   if (!wm_toolbar)
   {
@@ -652,9 +795,15 @@ void noskin_main_init (void)
     vbox = gtk_vbox_new (0, 0);
     gtk_container_add (GTK_CONTAINER (app), vbox);
     create_menus (app);
+
     gtk_box_pack_start (GTK_BOX (vbox), menubar, FALSE, FALSE, 0);
+    wm_show_hide_setup (menubar);
 
     gtk_container_add (GTK_CONTAINER (vbox), gtv);
+
+    w = gtk_vbox_new (0, 0);
+    gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
+    vbox = w;
   }
   else
   {
@@ -687,7 +836,7 @@ void noskin_main_init (void)
 
     g_object_connect (G_OBJECT (wm_toolbar),
         "signal::grab-focus", G_CALLBACK (appfocus_cb), NULL,
-	"signal::key-press-event", G_CALLBACK (keypress_cb), NULL,
+	/*"signal::key-press-event", G_CALLBACK (keypress_cb), NULL,*/
 	"signal::map-event", G_CALLBACK (window_wm_toolbar_snap), NULL,
 	"signal::delete-event", G_CALLBACK (wm_close_cb), NULL,
 	NULL);
@@ -704,16 +853,33 @@ void noskin_main_init (void)
 
   /* time_widget / infobar */
 
-  gtk_box_pack_start (GTK_BOX (vbox),
-		      widget_create_from_xml ("toolbar-window.xml",
-					      "toolbar_wm", NULL, FALSE),
-		      FALSE, FALSE, 0);
+  w = widget_create_from_xml ("toolbar-window.xml", "toolbar_wm",
+			      NULL, FALSE, TRUE);
+  if (!w)
+    exit (2); /* no point in continuing */
+  gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
+
+  if (!wm_toolbar)
+  {
+    wm_show_hide_setup (w);
+
+    /* hack: widget to take the focus in place of the video widget (which can't) */
+    w = gtk_label_new ("");
+    gtk_widget_set_size_request (w, 1, 1);
+    gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
+    gtk_label_set_selectable ((GtkLabel *)w, TRUE);
+    g_object_connect (G_OBJECT (w),
+	"signal::button_press_event", G_CALLBACK (gtk_true), NULL,
+	NULL);
+    gtk_widget_grab_focus (w);
+    wm_show_hide_setup (w);
+  }
 
   /* Show the app window (and ensure that it's resized properly) */
 
-  gtk_widget_show_all (app);
   gtk_window_resize (GTK_WINDOW (app),
 		     GTK_VIDEO_MIN_WIDTH, GTK_VIDEO_MIN_HEIGHT);
+  gtk_widget_show_all (app);
 
   if (wm_toolbar)
   {
@@ -724,6 +890,12 @@ void noskin_main_init (void)
 
   have_video = 1;
 
+  Display *d = GDK_WINDOW_XDISPLAY (app->window);
+  opacity_lock = XInternAtom (d, "_NET_WM_WINDOW_OPACITY_LOCKED", False);
+  XChangeProperty (d, GDK_WINDOW_XWINDOW (app->window), opacity_lock,
+		   XA_CARDINAL, 32, PropModeReplace,
+		   (unsigned char *)&have_video, 1);
+
   g_idle_add ((GSourceFunc) noskin_main_init_cb, NULL);
   g_timeout_add (500, update_slider_cb, NULL);
 }
