@@ -13,9 +13,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: play_item.h,v 1.11 2005/09/29 17:29:47 dsalt Exp $
+ * $Id: play_item.h,v 1.15 2006-03-05 23:57:01 dsalt Exp $
  *
  * playlist item / media mark:
  * an mrl + options (e.g. volume, start time, brightness/contrast ...)
@@ -30,6 +30,13 @@
 
 typedef struct play_item_s play_item_t;
 
+typedef enum {
+  PLAY_ITEM_NORMAL,
+  PLAY_ITEM_AUTOPLAY,
+  PLAY_ITEM_BROWSER,
+  PLAY_ITEM_PLAYLIST,
+} play_item_type;
+
 struct play_item_s {
 
   char   *title;
@@ -38,6 +45,7 @@ struct play_item_s {
   int     start_time;
 
   GList  *options; /* options are simply script engine command strings */
+  play_item_type type;
 
   gboolean untitled; /* the title was extracted from the MRL */
   gboolean played; /* this has been played in playlist */
@@ -63,6 +71,7 @@ void play_item_init (void);
 typedef enum {
   PLAY_ITEM_LIST,
   PLAY_ITEM_MEDIAMARK,
+  PLAY_ITEM_LIST_NEW,
   PLAY_ITEM_MEDIAMARK_NEW,
 } play_item_type_t;  
 int  play_item_edit (play_item_t *item, play_item_type_t type,
