@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2002 Stefan Holst
- * Copyright (C) 2003-2005 the xine project team
+ * Copyright (C) 2003-2006 the xine project team
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,9 +14,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: server.c,v 1.18 2005/10/14 17:54:44 dsalt Exp $
+ * $Id: server.c,v 1.22 2006/03/27 22:11:25 dsalt Exp $
  *
  * gxine unix domain socket remote control interface
  */
@@ -49,30 +49,31 @@ static int       gxsocket;
 static pthread_t server_thread;
 
 /* create an unix domain socket */
-static int make_socket (char *path) {
-
-  int                sock;
-  struct sockaddr_un adr = {0};
-
-  sock = socket (PF_UNIX, SOCK_STREAM, 0);
-  if (sock < 0) {
+static int make_socket (char *path)
+{
+  union {
+    struct sockaddr_un u;
+    struct sockaddr s;
+  } adr = { .u = {0} };
+  int sock = socket (PF_UNIX, SOCK_STREAM, 0);
+  if (sock < 0)
+  {
     perror ("socket");
     return -1;
-//    exit (EXIT_FAILURE);
   }
 
   unlink (path);
 
   /* fill socket address structures */
 
-  adr.sun_family = AF_UNIX;
-  strncpy (adr.sun_path, path, sizeof (adr.sun_path)-1);
+  adr.u.sun_family = AF_UNIX;
+  strncpy (adr.u.sun_path, path, sizeof (adr.u.sun_path)-1);
   
   /* bind the name to the descriptor */
-  if (bind (sock, (struct sockaddr *) &adr, sizeof (adr)) < 0) {
+  if (bind (sock, &adr.s, sizeof (adr)) < 0)
+  {
     perror ("bind");
     return -1;
-//    exit (EXIT_FAILURE);
   }
 
   logprintf ("server: socket '%s' created \n", path);
@@ -80,52 +81,16 @@ static int make_socket (char *path) {
   return sock;
 }
 
-#if 0
-/*
- * Check for socket validity.
- */
-static int sock_check_opened(int socket) {
-  fd_set   readfds, writefds, exceptfds;
-  int      retval;
-  struct   timeval timeout;
-  
-  for(;;) {
-    FD_ZERO(&readfds); 
-    FD_ZERO(&writefds); 
-    FD_ZERO(&exceptfds);
-    FD_SET(socket, &exceptfds);
-    
-    timeout.tv_sec  = 0; 
-    timeout.tv_usec = 0;
-    
-    retval = select(socket + 1, &readfds, &writefds, &exceptfds, &timeout);
-    
-    if(retval == -1 && (errno != EAGAIN && errno != EINTR))
-      return 0;
-    
-    if (retval != -1)
-      return 1;
-  }
+static void sock_write (int fd, char *msg)
+{
+  int len = strlen (msg);
+  int pos = 0;
 
-  return 0;
-}
-#endif 
+  while (pos < len)
+  {
+    int n = write (fd, msg+pos, len-pos);
 
-static void sock_write (int fd, char *msg) {
-
-  int len, pos;
-
-  len = strlen (msg);
-  pos = 0;
-
-  while (pos<len) {
-  
-    int n=0;
-
-    n = write (fd, msg+pos, len-pos);
-
-    if (n<=0) {
-      
+    if (n <= 0)
       switch (errno) {
       case EAGAIN:
 	continue;
@@ -133,14 +98,14 @@ static void sock_write (int fd, char *msg) {
 	perror ("server: write error");
 	return;
       }
-    }
-
-    pos += n;
+    else
+      pos += n;
   }
 }
 
-static void error_report (void *s, const char *msg, ...) {
-
+static void __attribute__ ((format (printf, 2, 3)))
+error_report (void *s, const char *msg, ...)
+{
   int fd = *(int *)s;
   char *amsg;
   va_list ap;
@@ -155,60 +120,60 @@ static void error_report (void *s, const char *msg, ...) {
 
 static int running;
 
-static void *socket_listener (void *data) {
+static __attribute__ ((noreturn)) void *socket_listener (void *data)
+{
+  listen (gxsocket, 1);
+  while (running)
+  {
+    struct sockaddr client;
+    socklen_t size;
+    char buf[BUF_SIZE + 1];
+    int s = accept (gxsocket, &client, &size);
 
-  struct sockaddr_un client;
-  socklen_t          size;
-  int                s;
-  char               buf[BUF_SIZE+1];
-  int                buf_len;
-  int                listening;
+    if (s == -1)
+    {
+      perror ("accept");
+      sleep (1);
+      continue;
+    }
 
-  listen (gxsocket,1);
-
-  while (running) {
-    s = accept (gxsocket, (struct sockaddr *) &client, &size);
     logprintf ("server: connected.\n");
     sock_write(s, _("This is gxine\n\nJavascript interface. Type 'help();' for help.\n"));
-    buf_len = 0;
 
-    listening = 1;
-    while (listening) {
-      int n;
+    int buf_len = 0;
+    int listening = 1;
 
-      n = read (s, buf+buf_len, 1);
-
-      if (n <= 0) {
-      
-	switch (errno) {
+    while (listening)
+    {
+      int n = read (s, buf + buf_len, 1);
+      if (n <= 0)
+      {
+	switch (errno)
+	{
 	case EAGAIN:
 	  continue;
-	case ECONNRESET:
-	  logprintf ("server: disconnected.\n");
-	  listening = 0;
-	  break;
 	default:
 	  if (errno)
-	    fprintf (stderr, _("server: read error %d\n"), errno);
+	    g_printerr (_("server: read error %d\n"), errno);
           else
-            printf (_("server: client disconnected\n"));
+        case ESPIPE:
+        case EPIPE:
+	case ECONNRESET:
+            g_print (_("server: client disconnected\n"));
 	  listening = 0;
 	  break;
 	}
-      } else {
-
-	if ( (buf[buf_len] == 10) || (buf[buf_len]==13) ) {
-	  buf[buf_len]=0;
-	  gdk_threads_enter();
-	  engine_exec_ext (buf, error_report, &s, NULL, NULL);
-	  gdk_threads_leave();
-	  
-	  buf_len = 0;
-	} else {
-	  if (buf_len<BUF_SIZE)
-	    buf_len += n;
-	}
       }
+      else if (buf[buf_len] == '\n' || buf[buf_len] == '\r')
+      {
+	buf[buf_len]=0;
+	gdk_threads_enter();
+	engine_exec_ext (buf, error_report, &s, NULL, NULL);
+	gdk_threads_leave();
+	buf_len = 0;
+      }
+      else if (buf_len < BUF_SIZE)
+	buf_len += n;
     }
     logprintf ("server: closing client connection.\n");
     close(s);
@@ -217,11 +182,10 @@ static void *socket_listener (void *data) {
   pthread_exit(NULL);
 }
 
-void server_setup (void) {
-
-  char filename [1024];
-  snprintf (filename, 1024, SOCKET_FILENAME, getenv ("HOME")); 
-
+void server_setup (void)
+{
+  char filename[FILENAME_MAX];
+  snprintf (filename, sizeof (filename), SOCKET_FILENAME, getenv ("HOME")); 
   gxsocket = make_socket (filename);
 }
 
@@ -229,8 +193,12 @@ void server_start (void)
 {
   if (gxsocket >= 0)
   {
+    pthread_attr_t attr;
     running = 1;
-    pthread_create(&server_thread, NULL, socket_listener, NULL);
+    pthread_attr_init (&attr);
+    pthread_attr_setstacksize (&attr, 64 * 1024); /* should be fine... */
+    pthread_create (&server_thread, &attr, socket_listener, NULL);
+    pthread_attr_destroy (&attr);
   }
 }
 
@@ -262,16 +230,19 @@ static void sigpipe_handler (int sn)
   signal (SIGPIPE, sigpipe_handler);
 }
 
-int server_client_connect (void) {
-
-  struct sockaddr_un cli_adr = {0}, serv_adr = {0};
+int server_client_connect (void)
+{
+  union {
+    struct sockaddr_un u;
+    struct sockaddr s;
+  } cli_adr = { .u = {0} }, serv_adr = { .u = {0} };
   int                length = sizeof(struct sockaddr_un);
-  char               filename [1024];
+  char               filename[PATH_MAX];
 
   signal (SIGPIPE, sigpipe_handler);
 
   /* server filename */
-  snprintf (filename, 1024, SOCKET_FILENAME, getenv ("HOME")); 
+  snprintf (filename, PATH_MAX, SOCKET_FILENAME, getenv ("HOME")); 
 
   logprintf ("server: trying to connect to already running instance of gxine (%s)...\n",
 	    filename);
@@ -279,23 +250,25 @@ int server_client_connect (void) {
   client_fd = socket (AF_UNIX, SOCK_STREAM, 0);
 
   /* initialize the client address structure */
-  cli_adr.sun_family = AF_UNIX;
+  cli_adr.u.sun_family = AF_UNIX;
  
   sprintf (tstr, "/tmp/gxine_%d", getpid ());
-  strcpy (cli_adr.sun_path, tstr);
+  strcpy (cli_adr.u.sun_path, tstr);
   unlink (tstr);
 
   /* bind the socket to the client's address */
-  if (bind (client_fd, (struct sockaddr*) &cli_adr, length) < 0) {
+  if (bind (client_fd, &cli_adr.s, length) < 0)
+  {
     perror ("bind");
     return 0;
   }
 
   /* initialize the server address structure */
-  serv_adr.sun_family = AF_UNIX;
-  strcpy (serv_adr.sun_path, filename);
+  serv_adr.u.sun_family = AF_UNIX;
+  strcpy (serv_adr.u.sun_path, filename);
 
-  if (connect (client_fd, (struct sockaddr *) &serv_adr, sizeof (serv_adr)) < 0) {
+  if (connect (client_fd, &serv_adr.s, sizeof (serv_adr)) < 0)
+  {
 #ifdef LOG
     perror ("connect");
 #endif
@@ -303,19 +276,19 @@ int server_client_connect (void) {
     return 0;
   }
   
-  printf (_("server: connected to existing gxine instance.\n"));
+  g_print (_("server: connected to existing gxine instance.\n"));
   atexit (server_client_stop);
 
   return 1;
 }
 
-void server_client_send (char *cmd) {
-
-  write (client_fd, cmd, strlen (cmd));
+int server_client_send (const char *cmd)
+{
+  return write (client_fd, cmd, strlen (cmd));
 }
 
-void server_client_stop (void) {
-
+void server_client_stop (void)
+{
   if (client_fd < 0)
     return;
 
